@@ -9,6 +9,8 @@
 //! ```
 //! use java_properties::PropertiesIter;
 //! use java_properties::PropertiesWriter;
+//! use java_properties::read;
+//! use java_properties::write;
 //! use std::collections::HashMap;
 //! use std::env::temp_dir;
 //! use std::fs::File;
@@ -21,23 +23,34 @@
 //! let mut file_name = temp_dir();
 //! file_name.push("java-properties-test.properties");
 //!
-//! // Writing
-//! let mut map1 = HashMap::new();
-//! map1.insert("a".to_string(), "b".to_string());
+//! // Writing simple
+//! let mut src_map1 = HashMap::new();
+//! src_map1.insert("a".to_string(), "b".to_string());
+//! let mut f = try!(File::create(&file_name));
+//! try!(write(BufWriter::new(f), &src_map1));
+//!
+//! // Writing advanced
+//! let mut src_map2 = HashMap::new();
+//! src_map2.insert("a".to_string(), "b".to_string());
 //! let mut f = try!(File::create(&file_name));
 //! let mut writer = PropertiesWriter::new(BufWriter::new(f));
-//! for (k, v) in map1.iter() {
+//! for (k, v) in src_map2.iter() {
 //!   try!(writer.write(&k, &v));
 //! }
 //! writer.flush();
 //!
-//! // Reading
+//! // Reading simple
+//! let mut f2 = try!(File::open(&file_name));
+//! let dst_map1 = try!(read(BufReader::new(f2)));
+//! assert_eq!(src_map1, dst_map1);
+//!
+//! // Reading advanced
 //! let mut f = try!(File::open(&file_name));
-//! let mut map2 = HashMap::new();
+//! let mut dst_map2 = HashMap::new();
 //! try!(PropertiesIter::new(BufReader::new(f)).read_into(|k, v| {
-//!   map2.insert(k, v);
+//!   dst_map2.insert(k, v);
 //! }));
-//! assert_eq!(map1, map2);
+//! assert_eq!(src_map2, dst_map2);
 //! # Ok(())
 //! # }
 //! # foo().unwrap();
@@ -54,13 +67,13 @@ use encoding::DecoderTrap;
 use encoding::RawEncoder;
 use encoding::all::ISO_8859_1;
 use regex::Regex;
+use std::collections::HashMap;
 use std::convert::From;
 use std::error::Error;
 use std::fmt;
 use std::fmt::Display;
 use std::fmt::Formatter;
 use std::io;
-use std::io::BufRead;
 use std::io::Bytes;
 use std::io::Read;
 use std::io::Write;
@@ -317,6 +330,18 @@ impl Line {
   }
 }
 
+impl Into<LineContent> for Line {
+  fn into(self) -> LineContent {
+    self.data
+  }
+}
+
+impl Display for Line {
+  fn fmt(&self, f: &mut Formatter) -> fmt::Result {
+    write!(f, "Line {{line_number: {}, content: {}}}", self.line_number, self.data)
+  }
+}
+
 /// Parsed content of the line.
 #[derive(PartialEq,Eq,PartialOrd,Ord,Debug)]
 pub enum LineContent {
@@ -327,6 +352,15 @@ pub enum LineContent {
   KVPair(String, String),
 }
 
+impl Display for LineContent {
+  fn fmt(&self, f: &mut Formatter) -> fmt::Result {
+    match self {
+      &LineContent::Comment(ref s) => write!(f, "Comment({:?})", s),
+      &LineContent::KVPair(ref k, ref v) => write!(f, "KVPair({:?}, {:?})", k, v),
+    }
+  }
+}
+
 /////////////////////
 
 fn unescape(s: &str, line_number: usize) -> Result<String, PropertiesError> {
@@ -443,7 +477,7 @@ impl LineParser {
   }
 }
 
-/// Parses a propeties file and iterates over its contents.
+/// Parses a properties file and iterates over its contents.
 ///
 /// For basic usage, see the crate-level documentation.
 /// Note that once `next` returns an error, the result of further calls is undefined.
@@ -527,13 +561,27 @@ fn unicode_escape(_encoder: &mut RawEncoder, input: &str, output: &mut ByteWrite
 
 static UNICODE_ESCAPE: EncoderTrap = EncoderTrap::Call(unicode_escape);
 
+/// A line ending style allowed in a Java properties file.
 #[derive(PartialEq,Eq,PartialOrd,Ord,Debug,Copy,Clone)]
 pub enum LineEnding {
+  /// Carriage return alone.
   CR,
+  /// Line feed alone.
   LF,
+  /// Carriage return followed by line feed.
   CRLF,
 }
 
+impl Display for LineEnding {
+  fn fmt(&self, f: &mut Formatter) -> fmt::Result {
+    match self {
+      &LineEnding::CR => f.write_str("LineEnding::CR"),
+      &LineEnding::LF => f.write_str("LineEnding::LF"),
+      &LineEnding::CRLF => f.write_str("LineEnding::CRLF"),
+    }
+  }
+}
+
 /// Writes to a properties file.
 pub struct PropertiesWriter<W: Write> {
   writer: W,
@@ -590,6 +638,8 @@ impl<W: Write> PropertiesWriter<W> {
         '\x0c' => escaped.push_str("\\f"),
         ':' => escaped.push_str("\\:"),
         '=' => escaped.push_str("\\="),
+        '!' => escaped.push_str("\\!"),
+        '#' => escaped.push_str("\\#"),
         _ if c < ' ' => escaped.push_str(&format!("\\u{:x}", c as u16)),
         _ => escaped.push(c), // We don't worry about other characters, since they're taken care of below.
       }
@@ -658,6 +708,32 @@ impl<W: Write> PropertiesWriter<W> {
 
 /////////////////////
 
+/// Writes a hash map to a properties file.
+///
+/// For more advanced use cases, use `PropertiesWriter`.
+pub fn write<W: Write>(writer: W, map: &HashMap<String, String>) -> Result<(), PropertiesError> {
+  let mut writer = PropertiesWriter::new(writer);
+  for (k, v) in map.iter() {
+    try!(writer.write(&k, &v));
+  }
+  try!(writer.flush());
+  Ok(())
+}
+
+/// Reads a properties file into a hash map.
+///
+/// For more advanced use cases, use `PropertiesIter`.
+pub fn read<R: Read>(input: R) -> Result<HashMap<String, String>, PropertiesError> {
+  let mut p = PropertiesIter::new(input);
+  let mut map = HashMap::new();
+  try!(p.read_into(|k, v| {
+    map.insert(k, v);
+  }));
+  Ok(map)
+}
+
+/////////////////////
+
 #[cfg(test)]
 mod tests {
   use encoding::all::ISO_8859_1;
@@ -827,6 +903,8 @@ mod tests {
       (r"", Some("")),
       (r"x", Some("x")),
       (r"\\", Some("\\")),
+      (r"\#", Some("#")),
+      (r"\!", Some("!")),
       (r"\\\n\r\t\f\u0001\b", Some("\\\n\r\t\x0c\u{0001}b")),
       (r"\", Some("\x00")),
       (r"\u", None),
@@ -856,6 +934,8 @@ mod tests {
     let data = [
       ("", vec![]),
       ("a=b", vec![mk_pair(1, "a", "b")]),
+      ("a=\\#b", vec![mk_pair(1, "a", "#b")]),
+      ("\\!a=b", vec![mk_pair(1, "!a", "b")]),
       ("a=b\nc=d\\\ne=f\ng=h\r#comment1\r\n#comment2\\\ni=j\\\n#comment3\n \n#comment4", vec![
         mk_pair(1, "a", "b"),
         mk_pair(2, "c", "de=f"),
@@ -890,6 +970,7 @@ mod tests {
       ("", "", "=\n"),
       ("a", "b", "a=b\n"),
       (" :=", " :=", "\\ \\:\\==\\ \\:\\=\n"),
+      ("!", "#", "\\!=\\#\n"),
       ("\u{1F41E}", "\u{1F41E}", "\\u1f41e=\\u1f41e\n"),
     ];
     for &(key, value, expected) in data.iter() {
@@ -1101,4 +1182,12 @@ mod tests {
     assert_eq!(format!("{}", PropertiesError::new("foo", None, None)), "foo (line_number = unknown)");
     assert_eq!(format!("{}", PropertiesError::new("foo", None, Some(1))), "foo (line_number = 1)");
   }
+
+  #[test]
+  fn line_display() {
+    assert_eq!(format!("{}", Line::mk_pair(1, "foo".to_string(), "bar".to_string())),
+               "Line {line_number: 1, content: KVPair(\"foo\", \"bar\")}");
+    assert_eq!(format!("{}", Line::mk_comment(1, "baz".to_string())),
+               "Line {line_number: 1, content: Comment(\"baz\")}");
+  }
 }
